#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_form.security module

"""

from pyramid.decorator import reify

from pyams_form.interfaces.form import IFormContextPermissionChecker, IFormSecurityContext

__docformat__ = 'restructuredtext'


def get_edit_permission(request, context=None):
    """Get required edit permission"""
    if context is None:
        context = request.context
    registry = request.registry
    checker = registry.queryMultiAdapter((context, request), IFormContextPermissionChecker)
    if checker is None:
        checker = registry.queryAdapter(context, IFormContextPermissionChecker)
    if checker is not None:
        return checker.edit_permission
    return None


class ProtectedFormObjectMixin:
    """Form object protected by a permission

    A "protected" form is a form on which you apply a permission; the context on which the security
    applies can be provided by an :py:class:`IFormSecurityContext
    <pyams_form.interfaces.IFormSecurityContext>` adapter, or will be extracted for the form
    context itself.

    The permission itself will be provided by an adapter to :py:class:`IFormContextPermissionChecker
    <pyams_form.interfaces.IFormContextPermissionChecker>`
    :

    This class is a form mixin class which should be used for forms protected by a
    security context.
    """

    @reify
    def permission(self):
        """This permission is required to be able to edit the form context"""
        request = self.request  # pylint: disable=no-member
        registry = request.registry
        checker = None
        context = IFormSecurityContext(self, None)
        if context is None:
            context = self.context  # pylint: disable=no-member
        view = getattr(self, '__parent__', None) or \
            getattr(self, 'view', None) or \
            getattr(self, 'table', None)
        if view is not None:
            checker = registry.queryMultiAdapter((context, request, view),
                                                 IFormContextPermissionChecker)
        if checker is None:
            checker = registry.queryAdapter(context, IFormContextPermissionChecker)
        if checker is not None:
            return checker.edit_permission
        return None
